@@ -30,13 +30,15 @@ import logging
 import os
 
 from plainbox import __version__ as plainbox_version
+from plainbox.i18n import gettext as _
 from plainbox.impl.applogic import PlainBoxConfig
 from plainbox.impl.commands import PlainBoxToolBase
 from plainbox.impl.commands.check_config import CheckConfigCommand
 from plainbox.impl.commands.dev import DevCommand
 from plainbox.impl.commands.run import RunCommand
-from plainbox.impl.commands.startprovider import StartProviderCommand
 from plainbox.impl.commands.selftest import SelfTestCommand
+from plainbox.impl.commands.session import SessionCommand
+from plainbox.impl.commands.startprovider import StartProviderCommand
 from plainbox.impl.logging import setup_logging
 
 
@@ -62,6 +64,13 @@ class PlainBoxTool(PlainBoxToolBase):
         """
         return cls.format_version_tuple(plainbox_version)
 
+    def create_parser_object(self):
+        parser = super().create_parser_object()
+        parser.prog = "plainbox"
+        parser.usage = _("plainbox [--help] [--version] | [options] <command>"
+                         " ...")
+        return parser
+
     def add_subcommands(self, subparsers):
         """
         Add top-level subcommands to the argument parser.
@@ -72,6 +81,7 @@ class PlainBoxTool(PlainBoxToolBase):
         # TODO: switch to plainbox plugins
         RunCommand(self._provider_list, self._config).register_parser(
             subparsers)
+        SessionCommand().register_parser(subparsers)
         SelfTestCommand().register_parser(subparsers)
         CheckConfigCommand(self._config).register_parser(subparsers)
         DevCommand(self._provider_list, self._config).register_parser(
@@ -99,6 +109,10 @@ def main(argv=None):
     raise SystemExit(PlainBoxTool().main(argv))
 
 
+def get_parser_for_sphinx():
+    return PlainBoxTool().construct_parser()
+
+
 # Setup logging before anything else starts working.
 # If we do it in main() or some other place then unit tests will see
 # "leaked" log files which are really closed when the runtime shuts
