@@ -50,15 +50,16 @@ from plainbox.i18n import gettext as _
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyMissingError
 from plainbox.impl.job import JobDefinition
-from plainbox.impl.job import JobOutputTextSource
 from plainbox.impl.resource import ExpressionCannotEvaluateError
 from plainbox.impl.resource import ExpressionFailedError
 from plainbox.impl.resource import Resource
 from plainbox.impl.secure.config import Unset
+from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
 from plainbox.impl.secure.rfc822 import gen_rfc822_records
 from plainbox.impl.session.jobs import JobReadinessInhibitor
+from plainbox.vendor import extcmd
 
 __all__ = [
     'CheckBoxSessionStateController',
@@ -277,10 +278,10 @@ class CheckBoxSessionStateController(ISessionStateController):
                 logger.warning(
                     # TRANSLATORS: keep the word "local" untranslated. It is a
                     # special type of job that needs to be distinguished.
-                    _("Local job %s produced job %r that collides with"
+                    _("Local job %s produced job %s that collides with"
                       " an existing job %s (from %s), the new job was"
                       " discarded"),
-                    job, exc.duplicate_job, exc.job, exc.job.origin)
+                    job.id, exc.duplicate_job.id, exc.job.id, exc.job.origin)
             else:
                 # Patch the origin of the existing job so that it traces
                 # back to the job that "generated" it again. This is
@@ -511,13 +512,15 @@ class CheckBoxExecutionController(IExecutionController):
         """
         # Get a proper environment
         env = dict(os.environ)
-        # Use non-internationalized environment
-        env['LANG'] = 'C.UTF-8'
-        if 'LANGUAGE' in env:
-            del env['LANGUAGE']
-        for name in list(env.keys()):
-            if name.startswith("LC_"):
-                del env[name]
+        # Neuter locale unless 'preserve-locale' flag is set
+        if 'preserve-locale' not in job.get_flag_set():
+            # Use non-internationalized environment
+            env['LANG'] = 'C.UTF-8'
+            if 'LANGUAGE' in env:
+                del env['LANGUAGE']
+            for name in list(env.keys()):
+                if name.startswith("LC_"):
+                    del env[name]
         # Use PATH that can lookup checkbox scripts
         if job.provider.extra_PYTHONPATH:
             env['PYTHONPATH'] = os.pathsep.join(
@@ -527,15 +530,18 @@ class CheckBoxExecutionController(IExecutionController):
         env['PATH'] = os.pathsep.join(
             [nest_dir]
             + env.get("PATH", "").split(os.pathsep))
-        # Add CHECKBOX_SHARE that is needed by one script
-        if job.provider.CHECKBOX_SHARE is not None:
-            env['CHECKBOX_SHARE'] = job.provider.CHECKBOX_SHARE
-        # Add CHECKBOX_DATA (temporary checkbox data)
+        # Add per-session shared state directory
+        env['PLAINBOX_SESSION_SHARE'] = self.CHECKBOX_DATA
         env['CHECKBOX_DATA'] = self.CHECKBOX_DATA
-        # Add plainbox equivalents of the two above
+        # Add a path to the per-provider data directory
         if job.provider.data_dir is not None:
             env['PLAINBOX_PROVIDER_DATA'] = job.provider.data_dir
-        env['PLAINBOX_SESSION_SHARE'] = self.CHECKBOX_DATA
+        # Add a path to the per-provider units directory
+        if job.provider.units_dir is not None:
+            env['PLAINBOX_PROVIDER_UNITS'] = job.provider.units_dir
+        # Add a path to the base provider directory (legacy)
+        if job.provider.CHECKBOX_SHARE is not None:
+            env['CHECKBOX_SHARE'] = job.provider.CHECKBOX_SHARE
         # Inject additional variables that are requested in the config
         if config is not None and config.environment is not Unset:
             for env_var in config.environment:
@@ -559,6 +565,14 @@ class CheckBoxExecutionController(IExecutionController):
         """
         return os.path.join(self._session_dir, "CHECKBOX_DATA")
 
+    def get_warm_up_for_job(self, job):
+        """
+        Get a warm-up function that should be called before running this job.
+
+        :returns:
+            None
+        """
+
 
 class UserJobExecutionController(CheckBoxExecutionController):
     """
@@ -580,9 +594,9 @@ class UserJobExecutionController(CheckBoxExecutionController):
         :returns:
             List of command arguments
 
-        This basically returns ['bash', '-c', job.command]
+        This basically returns [job.shell, '-c', job.command]
         """
-        return ['bash', '-c', job.command]
+        return [job.shell, '-c', job.command]
 
     def get_checkbox_score(self, job):
         """
@@ -745,6 +759,9 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         # Only works with jobs loaded from the secure PROVIDERPATH
         if not job.provider.secure:
             return -1
+        # Doesn't work when connected over SSH (LP: #1299201)
+        if os.environ.get("SSH_CONNECTION"):
+            return -1
         # Only makes sense with jobs that need to run as another user
         # Promote this controller only if the trusted launcher is authorized to
         # run jobs as another user
@@ -753,6 +770,30 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         else:
             return 0
 
+    def get_warm_up_for_job(self, job):
+        """
+        Get a warm-up function that should be called before running this job.
+
+        :returns:
+            a warm-up function for jobs that need to run as another
+            user or None if the job can run as the current user.
+        """
+        if job.user is None:
+            return
+        else:
+            return plainbox_trusted_launcher_warm_up
+
+
+def plainbox_trusted_launcher_warm_up():
+    """
+    Warm-up function for plainbox-trusted-laucher-1.
+
+    returned by :meth:`RootViaPTL1ExecutionController.get_warm_up_for_job()`
+    """
+    warmup_popen = extcmd.ExternalCommand()
+    return warmup_popen.call(
+        ['pkexec', 'plainbox-trusted-launcher-1', '--warmup'])
+
 
 class RootViaPkexecExecutionController(
         CheckBoxDifferentialExecutionController):
@@ -795,8 +836,8 @@ class RootViaPkexecExecutionController(
             job, config, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
-        # Lastly use bash -c, to run our command
-        cmd += ['bash', '-c', job.command]
+        # Lastly use job.shell -c, to run our command
+        cmd += [job.shell, '-c', job.command]
         return cmd
 
     def get_checkbox_score(self, job):
@@ -880,8 +921,8 @@ class RootViaSudoExecutionController(
             job, config, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
-        # Lastly use bash -c, to run our command
-        cmd += ['bash', '-c', job.command]
+        # Lastly use job.shell -c, to run our command
+        cmd += [job.shell, '-c', job.command]
         return cmd
 
     def get_checkbox_score(self, job):
