@@ -42,11 +42,11 @@ from plainbox.impl.ctrl import UserJobExecutionController
 from plainbox.impl.ctrl import gen_rfc822_records_from_io_log
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.job import JobDefinition
-from plainbox.impl.job import JobOutputTextSource
 from plainbox.impl.resource import Resource
 from plainbox.impl.resource import ResourceExpression
+from plainbox.impl.secure.origin import JobOutputTextSource
+from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.providers.v1 import Provider1
-from plainbox.impl.secure.rfc822 import Origin
 from plainbox.impl.secure.rfc822 import RFC822Record
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
 from plainbox.impl.session import JobReadinessInhibitor
@@ -362,10 +362,10 @@ class CheckBoxSessionStateControllerTests(TestCase):
             job, result)
         # Ensure that a warning was logged
         mock_logger.warning.assert_called_once_with(
-            ("Local job %s produced job %r that collides with"
+            ("Local job %s produced job %s that collides with"
              " an existing job %s (from %s), the new job was"
              " discarded"),
-            job, clashing_job, existing_job, existing_job.origin)
+            job.id, clashing_job.id, existing_job.id, existing_job.origin)
 
     @mock.patch('plainbox.impl.ctrl.gen_rfc822_records_from_io_log')
     def test_observe_result__local_perfect_clash(self, mock_gen):
@@ -532,7 +532,9 @@ class CheckBoxExecutionControllerTestsMixIn:
                     (IProviderBackend1, IProvider1), {}),
                 extra_PYTHONPATH=None,
                 CHECKBOX_SHARE='CHECKBOX_SHARE',
-                data_dir='data_dir'))
+                data_dir='data_dir', units_dir='units_dir'))
+        # Mock the default flags (empty set)
+        self.job.get_flag_set.return_value = frozenset()
         # Create mocked config.
         # Put an empty dictionary of environment overrides
         # that is expected by get_execution_environment()
@@ -617,12 +619,12 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
 
     def test_get_command(self):
         """
-        verify that we simply execute the command via bash
+        verify that we simply execute the command via job.shell
         """
         self.assertEqual(
             self.ctrl.get_execution_command(
                 self.job, self.config, self.NEST_DIR),
-            ['bash', '-c', self.job.command])
+            [self.job.shell, '-c', self.job.command])
 
     def test_get_checkbox_score_for_jobs_without_user(self):
         """
@@ -655,9 +657,18 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         # Call the tested method
         env = self.ctrl.get_execution_environment(
             self.job, self.config, self.NEST_DIR)
-        # Ensure that LANG is rese to C.UTF-8
+        # Ensure that LANG is reset to C.UTF-8
         self.assertEqual(env['LANG'], 'C.UTF-8')
 
+    @mock.patch.dict('os.environ', clear=True, LANG='fake_LANG')
+    def test_get_execution_environment_preserves_LANG_if_requested(self):
+        self.job.get_flag_set.return_value = {'preserve-locale'}
+        # Call the tested method
+        env = self.ctrl.get_execution_environment(
+            self.job, self.config, self.NEST_DIR)
+        # Ensure that LANG is what we mocked it to be
+        self.assertEqual(env['LANG'], 'fake_LANG')
+
     @mock.patch.dict('os.environ', clear=True, PYTHONPATH='PYTHONPATH')
     def test_get_execution_environment_keeps_PYTHONPATH(self):
         # Call the tested method
@@ -754,7 +765,10 @@ class RootViaPTL1ExecutionControllerTests(
                 spec=IProviderBackend1,
                 extra_PYTHONPATH=None,
                 data_dir="data_dir-generator",
+                units_dir="units_dir-generator",
                 CHECKBOX_SHARE='CHECKBOX_SHARE-generator'))
+        # Mock the default flags (empty set)
+        self.job.origin.source.job.get_flag_set.return_value = frozenset()
         PATH = os.pathsep.join([self.NEST_DIR, 'vanilla-path'])
         expected = [
             'pkexec', '--user', self.job.user,
@@ -765,6 +779,7 @@ class RootViaPTL1ExecutionControllerTests(
             '-G', 'LANG=C.UTF-8',
             '-G', 'PATH={}'.format(PATH),
             '-G', 'PLAINBOX_PROVIDER_DATA=data_dir-generator',
+            '-G', 'PLAINBOX_PROVIDER_UNITS=units_dir-generator',
             '-G', 'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
             '--target', self.job.checksum,
             '-T', 'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
@@ -772,6 +787,7 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'LANG=C.UTF-8',
             '-T', 'PATH={}'.format(PATH),
             '-T', 'PLAINBOX_PROVIDER_DATA=data_dir',
+            '-T', 'PLAINBOX_PROVIDER_UNITS=units_dir',
             '-T', 'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
         ]
         actual = self.ctrl.get_execution_command(
@@ -795,6 +811,7 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'LANG=C.UTF-8',
             '-T', 'PATH={}'.format(PATH),
             '-T', 'PLAINBOX_PROVIDER_DATA=data_dir',
+            '-T', 'PLAINBOX_PROVIDER_UNITS=units_dir',
             '-T', 'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
         ]
         actual = self.ctrl.get_execution_command(
@@ -814,6 +831,7 @@ class RootViaPTL1ExecutionControllerTests(
         # Ensure that we get a negative score of minus one
         self.assertEqual(self.ctrl.get_checkbox_score(self.job), -1)
 
+    @mock.patch.dict('plainbox.impl.ctrl.os.environ', clear=True)
     def test_get_checkbox_score_for_secure_provider_and_user_job(self):
         # Assume that the job is coming from Provider1 provider
         # and the provider is secure
@@ -823,6 +841,7 @@ class RootViaPTL1ExecutionControllerTests(
         # Ensure that we get a neutral score of zero
         self.assertEqual(self.ctrl.get_checkbox_score(self.job), 0)
 
+    @mock.patch.dict('plainbox.impl.ctrl.os.environ', clear=True)
     @mock.patch('plainbox.impl.ctrl.check_output')
     def test_get_checkbox_score_for_secure_provider_root_job_with_policy(
             self, mock_check_output):
@@ -838,6 +857,7 @@ class RootViaPTL1ExecutionControllerTests(
         ctrl = self.CLS(self.SESSION_DIR, self.PROVIDER_LIST)
         self.assertEqual(ctrl.get_checkbox_score(self.job), 3)
 
+    @mock.patch.dict('plainbox.impl.ctrl.os.environ', clear=True)
     @mock.patch('plainbox.impl.ctrl.check_output')
     def test_get_checkbox_score_for_secure_provider_root_job_with_policy_2(
             self, mock_check_output):
@@ -855,6 +875,27 @@ class RootViaPTL1ExecutionControllerTests(
         ctrl = self.CLS(self.SESSION_DIR, self.PROVIDER_LIST)
         self.assertEqual(ctrl.get_checkbox_score(self.job), 3)
 
+    @mock.patch.dict('plainbox.impl.ctrl.os.environ', values={
+        'SSH_CONNECTION': '1.2.3.4 123 1.2.3.5 22'
+    })
+    @mock.patch('plainbox.impl.ctrl.check_output')
+    def test_get_checkbox_score_for_normally_supported_job_over_ssh(
+            self, mock_check_output):
+        # Assume that the job is coming from Provider1 provider
+        # and the provider is secure
+        self.job.provider = mock.Mock(spec=Provider1, secure=True)
+        # Assume that the job runs as root
+        self.job.user = 'root'
+        # Assume we get the right action id from pkaction(1) even with
+        # polikt version < 0.110 (pkaction always exists with status 1), see:
+        # https://bugs.freedesktop.org/show_bug.cgi?id=29936#attach_78263
+        mock_check_output.side_effect = CalledProcessError(
+            1, '', b"org.freedesktop.policykit.pkexec.run-plainbox-job\n")
+        # Ensure that we get a positive score of three
+        ctrl = self.CLS(self.SESSION_DIR, self.PROVIDER_LIST)
+        self.assertEqual(ctrl.get_checkbox_score(self.job), -1)
+
+    @mock.patch.dict('plainbox.impl.ctrl.os.environ', clear=True)
     @mock.patch('plainbox.impl.ctrl.check_output')
     def test_get_checkbox_score_for_secure_provider_root_job_no_policy(
             self, mock_check_output):
@@ -888,7 +929,7 @@ class RootViaPkexecExecutionControllerTests(
     @mock.patch.dict('os.environ', clear=True, PATH='vanilla-path')
     def test_get_command(self):
         """
-        verify that we run env(1) + bash(1) as the target user
+        verify that we run env(1) + job.shell as the target user
         """
         self.job.get_environ_settings.return_value = []
         self.assertEqual(
@@ -902,8 +943,9 @@ class RootViaPkexecExecutionControllerTests(
              'PATH={}'.format(
                  os.pathsep.join([self.NEST_DIR, 'vanilla-path'])),
              'PLAINBOX_PROVIDER_DATA=data_dir',
+             'PLAINBOX_PROVIDER_UNITS=units_dir',
              'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
-             'bash', '-c', self.job.command])
+             self.job.shell, '-c', self.job.command])
 
     def test_get_checkbox_score_for_user_jobs(self):
         # Assume that the job runs as the current user
@@ -942,8 +984,9 @@ class RootViaSudoExecutionControllerTests(
              'PATH={}'.format(
                  os.pathsep.join([self.NEST_DIR, 'vanilla-path'])),
              'PLAINBOX_PROVIDER_DATA=data_dir',
+             'PLAINBOX_PROVIDER_UNITS=units_dir',
              'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
-             'bash', '-c', self.job.command])
+             self.job.shell, '-c', self.job.command])
 
     SUDO, ADMIN = range(2)
 
